#! /bin/bash
# Install script, (C) AbsInt Angewandte Informatik GmbH


### customizable stuff
NAME="aiSee"
VERSION="3.4"
BUILD="175811"
PRODUCT="aiSee"
LICENSEFILE="./share/LICENSE"
INSTALL_LOG="install_log.txt"

### you shouldn't have to change anything below this line
F_BIN="aisee3"
F_LIBEXEC=`find bin/ | grep -v "^bin/\$" | cut -c 5-`
F_SHARE=`find share/ | grep -v "^share/\$" | cut -c 7-`
F_DOC=`find help/ | grep -v "^help/\$" | cut -c 6-`

if test `uname` = "Linux" ; then
    ECHOPTS="-e"
else
    ECHOPTS=""
fi

### abort the installation process
abort()
{
    echo ""
    echo "*** Installation was aborted due to problems! ***"
    echo "*** See '$INSTALL_LOG' for details.           ***"
    exit 1
}

### ask for yes/no answer
### if $1 is empty, NO is the default answer, otherwise YES is default
yesno()
{
    if test "$1" = "" ; then
        echo $ECHOPTS "(Yes/[No]) \c"
        read answer
        if test "" = "$answer" ; then
            answer="no"
        fi
    else
        echo $ECHOPTS "([Yes]/No) \c"
        read answer
        if test "" = "$answer" ; then
            answer="yes"
        fi
    fi
    if test "Yes" != $answer -a "yes" != $answer -a "Y" != $answer -a "y" != $answer; then
        echo "Installation aborted by user request."
        exit 2
    fi
}

### $1 is the target-path for installation
### $2 is the source-path for installation
### $3 is the name of a file or directory to be installed
test_install()
{
    source="$2/$3"
    target="$1/$3"

    # files
    if ! test -d "$source" ; then
        if test -e "$target"; then
            overwritten=`echo "${overwritten}\n${target}"`
        fi
    fi
}

### $1 is the target-path for installation
### $2 is the source-path for installation
### $3 is the name of a file or directory to be installed
### $4 is the permission mode in case $2 is a file
do_install()
{
    source="$2/$3"
    target="$1/$3"

    # directories
    if test -d "$source" ; then
        echo " - trying to create directory '$target'" >> $INSTALL_LOG
        mkdir -p "$target"  2>&1 >> $INSTALL_LOG || abort
        chmod 755 "$target" 2>&1 >> $INSTALL_LOG || abort
        echo "   OK" >> $INSTALL_LOG
    # files
    else
        echo " - trying to copy file '$source' to '$target'" >> $INSTALL_LOG
        cp -df "$source" "$target" 2>&1 >> $INSTALL_LOG || abort
        if test -f "$target" ; then
          chmod $4 "$target"        2>&1 >> $INSTALL_LOG || abort
        fi
        echo "   OK" >> $INSTALL_LOG
    fi
}



### ask for an installation path
### $1 is the default
### $2 is a description for the user
### the result is returned in $ask_path
ask_path()
{
    default_path=$1
    echo $2
    echo $ECHOPTS "    [$default_path] \c"
    read ask_path
    if test "" = "$ask_path" ; then
        ask_path="$default_path"
    fi
    real_path="`(cd \"$ask_path\" && pwd) 2> /dev/null`"
    if test "" != "$real_path"; then
      ask_path="$real_path"
    fi
}

### welcome message
echo
echo "This shell script will install $NAME on your system"
echo

### start writing log file
echo "*** Starting installation of $NAME ***" >> $INSTALL_LOG
echo ""                                       >> $INSTALL_LOG

### license agreement
while true ; do
    echo "Do you agree to the conditions as described in the license agreement?"
    echo "Press RETURN to read the Licensefile..."
    echo $ECHOPTS "(Yes/No/RETURN) \c"
    read agreed
    if test "yes" = "$agreed" || test "Yes" = "$agreed" || test "YES" = "$agreed" || test "y" = "$agreed" || test "Y" = "$agreed"; then
        echo ""                                >> $INSTALL_LOG
        echo "User agreed to license details." >> $INSTALL_LOG
        break;
    else
        if test "no" = "$agreed" || test "No" = "$agreed" || test "NO" = "$agreed" || test "n" = "$agreed" || test "N" = "$agreed" ; then
            echo ""                                          >> $INSTALL_LOG
            echo "User denied agreement to license details." >> $INSTALL_LOG
            echo "Installation aborted by user request"
            abort
        fi
    fi
    less $LICENSEFILE
    echo
done
echo

### installation paths
if test "" = "$AI_DEFAULT_PATH" ; then
  ai_default_path="/usr/local"
else
  ai_default_path="$AI_DEFAULT_PATH"
fi

ask_path "${ai_default_path}" "Enter the base path for your installation"
base_path=`eval echo $ask_path`
echo

ask_path "${base_path}/bin" "Where do you want the wrapper scripts for the executables to be installed?"
exec_path=`eval echo $ask_path`
echo

ask_path "${base_path}/share/${PRODUCT}" "Where do you want the product to be installed?"
product_path=`eval echo $ask_path`
echo

### show configuration
echo "Your configuration is as follows:"
echo "  - $NAME will be installed"
echo "  - Executable wrappers will be installed in '$exec_path'"
echo "  - Product will be installed in '$product_path'"
echo

### ask for confirmation
echo $ECHOPTS "Are these settings correct? \c"
yesno 1
echo

echo ""                                                             >> $INSTALL_LOG
echo "Using following configuration:"                               >> $INSTALL_LOG
echo "  - installing $NAME"                                         >> $INSTALL_LOG
echo "  - installing executable wrappers in '$exec_path'"           >> $INSTALL_LOG
echo "  - installing product in '$product_path'"                    >> $INSTALL_LOG

### instanciate the wrappers
mkdir -p binwrappers
sed "s#@EXECUTABLE@#$product_path/bin/aisee3#g" wrapper.sh > binwrappers/aisee3 || abort

### test for existing files
overwritten=""

for f in $F_BIN ; do
    test_install "$exec_path" "binwrappers" "$f" 755
done

for f in $F_LIBEXEC ; do
    test_install "$product_path" "bin" "$f" 755
done

for f in $F_SHARE ; do
    test_install "$product_path" "share" "$f" 644
done

for f in $F_DOC ; do
    test_install "$product_path" "help" "$f" 644
done

if test "" != "${overwritten}"; then

    while true ; do

        echo $ECHOPTS "WARNING: there are files that would be overwritten by the install script!"
        echo $ECHOPTS "Press RETURN to see the list of files, enter YES to start the installation"
        echo $ECHOPTS "or NO if you want to abort..."
        echo $ECHOPTS "(Yes/No/RETURN) \c"
        read agreed
        echo

        if test "yes" = "$agreed" || test "Yes" = "$agreed" || test "YES" = "$agreed" ; then
            echo ""                                                             >> $INSTALL_LOG
            echo "User agreed to overwrite existing files during installation." >> $INSTALL_LOG
            break;
        else
            if test "no" = "$agreed" || test "No" = "$agreed" || test "NO" = "$agreed" ; then
                echo ""                                                                  >> $INSTALL_LOG
                echo "Installation aborted since existing files may not be overwritten." >> $INSTALL_LOG
                echo "Installation aborted by user request"
                exit 1
            fi
        fi
        echo $ECHOPTS "${overwritten}" | less
    done
fi

### install files
echo "Copying files..."
echo

echo ""                        >> $INSTALL_LOG
echo "Installing executable wrappers." >> $INSTALL_LOG
mkdir -p "$exec_path" 2>&1 >> $INSTALL_LOG || abort
for f in $F_BIN ; do
    do_install "$exec_path" "binwrappers" "$f" 755
done

echo ""                                      >> $INSTALL_LOG
echo "Installing supplementary executables." >> $INSTALL_LOG
mkdir -p "$product_path/bin" 2>&1 >> $INSTALL_LOG || abort
for f in $F_LIBEXEC ; do
    do_install "$product_path/bin" "bin" "$f" 755
done

echo ""                         >> $INSTALL_LOG
echo "Installing shared files." >> $INSTALL_LOG
mkdir -p "$product_path/share" 2>&1 >> $INSTALL_LOG || abort
for f in $F_SHARE ; do
    do_install "$product_path/share" "share" "$f" 644
done

echo ""                                >> $INSTALL_LOG
echo "Installing documentation files." >> $INSTALL_LOG
mkdir -p "$product_path/help" 2>&1 >> $INSTALL_LOG || abort
for f in $F_DOC ; do
    do_install "$product_path/help" "help" "$f" 644
done

### inform about successful installation
echo "$NAME has been successfully installed."
echo
echo "You can start $NAME via the following command:"
echo "  $exec_path/aisee3"
echo
echo ""                                      >> $INSTALL_LOG
echo "** Installation finished successfully" >> $INSTALL_LOG

## end of file
